@@ -7,7 +7,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  * 
- * Copyrigth (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
  */
 #include "globals.h"
 
@@ -872,7 +872,8 @@ xmlSecEncCtxEncDataNodeRead(xmlSecEncCtxPtr encCtx, xmlNodePtr node) {
     }	
 
     /* TODO: KeyInfo node != NULL and encKey != NULL */    	
-    if((encCtx->encKey == NULL) && (encCtx->keyInfoReadCtx.keysMngr->getKey != NULL)) {
+    if((encCtx->encKey == NULL) && (encCtx->keyInfoReadCtx.keysMngr != NULL) 
+			&& (encCtx->keyInfoReadCtx.keysMngr->getKey != NULL)) {
 	encCtx->encKey = (encCtx->keyInfoReadCtx.keysMngr->getKey)(encCtx->keyInfoNode, 
 							     &(encCtx->keyInfoReadCtx));
     }
@@ -1078,6 +1079,7 @@ xmlSecEncCtxCipherReferenceNodeRead(xmlSecEncCtxPtr encCtx, xmlNodePtr node) {
 void 
 xmlSecEncCtxDebugDump(xmlSecEncCtxPtr encCtx, FILE* output) {
     xmlSecAssert(encCtx != NULL);
+    xmlSecAssert(output != NULL);
 
     switch(encCtx->mode) {
 	case xmlEncCtxModeEncryptedData:
@@ -1161,6 +1163,7 @@ xmlSecEncCtxDebugDump(xmlSecEncCtxPtr encCtx, FILE* output) {
 void 
 xmlSecEncCtxDebugXmlDump(xmlSecEncCtxPtr encCtx, FILE* output) {
     xmlSecAssert(encCtx != NULL);
+    xmlSecAssert(output != NULL);
 
     switch(encCtx->mode) {
 	case xmlEncCtxModeEncryptedData:
